import os

from bootstrapbase import BootstrapBase
from cluster_info import ClusterInfo
from common.const import Constants
from common.mapr_logger.log import Log
from operations.operationsbase import OperationsBase

from operations.shared import SharedSystem
from operations.csi import CSI
from operations.dataplatform import DataPlatform
from operations.ldap import LDAP
from operations.compute import Compute
from operations.kubeflow import Kubeflow
from operations.nodesvc import Nodesvc
from operations.spark import Spark
from operations.autoticket_generator import AutoTicketGenerator


class BootstrapUpgrade(BootstrapBase):
    def __init__(self):
        super(BootstrapUpgrade, self).__init__(BootstrapBase.UPGRADE)
        self._parse_args()

    def run(self):
        super(BootstrapUpgrade, self).run()
        k8s = OperationsBase()
        k8s.load_replace_dict()
        shared = SharedSystem()
        nodesvc = Nodesvc()
        csi = CSI()
        ldap = LDAP(self._prompts)
        dataplatform = DataPlatform()
        compute = Compute()
        spark = Spark()
        autoticket_generator = AutoTicketGenerator()
        kubeflow = Kubeflow()

        self.prologue()
        self.python_check()

        cluster_info = ClusterInfo()

        do_airgap = True
        do_compute = True
        do_kubeflow = False
        install_kubeflow = False
        do_spark = False
        install_spark = True
        install_autoticket_generator = True

        self.check_laptop_tools()
        self.configure_kubernetes()
        cluster_info.examine_cluster(False)

        print("")
        Log.info("Cluster details:", stdout=True)
        install_storage = cluster_info.is_data_fabric_installed()
        Log.info("  Data Fabric Installed: " + str(install_storage), stdout=True)
        install_compute = cluster_info.is_compute_installed()
        Log.info("  Compute Installed: " + str(install_compute), stdout=True)
        install_exampleldap = cluster_info.is_ldap_installed()
        Log.info("  Example LDAP Installed: " + str(install_exampleldap), stdout=True)
        install_csi = cluster_info.is_csi_installed()
        Log.info("  CSI Installed: " + str(install_csi), stdout=True)

        if do_compute:
            if do_spark:
                install_spark = cluster_info.is_spark_installed()
            if do_kubeflow:
                install_kubeflow = self.check_if_kubeflow()

        if do_airgap:
            if self.check_if_airgapped():
                self.airgap_location()

        OperationsBase.replace_dict["{operator-repo}"] = OperationsBase.operator_repo
        OperationsBase.replace_dict["{csi-repo}"] = OperationsBase.csi_repo
        OperationsBase.replace_dict["{kdf-repo}"] = OperationsBase.kdf_rep
        OperationsBase.replace_dict["{kubeflow-repo}"] = OperationsBase.kubeflow_repo
        OperationsBase.replace_dict["{local-path-provisioner-repo}"] = OperationsBase.local_path_provisioner_repo
        OperationsBase.replace_dict["{kfctl-hcp-istio-repo}"] = OperationsBase.kfctl_hcp_istio_repo
        OperationsBase.replace_dict["{fake-labels}"] = "true"

        if not self.check_ready():
            return

        nodesvc.install_nodesvc(True)
        shared.install_common_components(True)

        if install_exampleldap:
            ldap.install_exampleldap(True)

        if install_storage:
            dataplatform.install_dataplatform(True)

        if install_csi:
            csi.install_csi_components(True)

        if install_compute:
            compute.install_compute_components(True)
            if install_spark:
                spark.install_spark_components(True)
            if install_autoticket_generator:
                autoticket_generator.run_install(True)
            if install_kubeflow:
                kubeflow.install_kubeflow_components(True)

        print("")
        cluster_info.examine_cluster(True)
        Log.info(str(cluster_info), stdout=True)

        self.complete_installation()

    def check_if_airgapped(self):
        print(os.linesep)
        agree = self._prompts.prompt_boolean("Use Airgapped Docker Registry? Note: All bootstrap containers must exist in airgap registry!", False, key_name="USE_AIRGAP")
        return agree

    def airgap_location(self):
        print(os.linesep)
        registry = self._prompts.prompt("Airgapped Registry Location?", Constants.OPERATOR_REPO, False, key_name="AIRGAP_REGISTRY")
        OperationsBase.csi_repo = registry
        OperationsBase.kdf_rep = registry
        OperationsBase.kubeflow_repo = registry
        OperationsBase.operator_repo = registry
        OperationsBase.kfctl_hcp_istio_repo = registry

    def check_if_csi(self):
        choice = self._prompts.prompt_boolean("Update MapR CSI driver?", True, key_name="INSTALL_CSI")
        return choice

    def check_if_spark(self):
        # TODO: Replace with code to sense existing operator and offer to upgrade
        choice = self._prompts.prompt_boolean("Update Spark Operator?", True, key_name="INSTALL_SPARK")
        return choice

    def check_if_kubeflow(self):
        # TODO: Add code to sense existing operator and upgrade if avail. Keep question
        choice = self._prompts.prompt_boolean("Update Kubeflow Operator?", False, key_name="INSTALL_KUBEFLOW")
        return choice


if __name__ == '__main__':
    bootstrap_upgrade = BootstrapUpgrade()
    try:
        bootstrap_upgrade.run()
    except Exception as e:
        Log.exception(e)
        raise e
    BootstrapBase.exit_application(0)
