import json
import os
import time

from common.mapr_logger.log import Log
from common.os_command import OSCommand


class ClusterInfo(object):
    SYSTEM = "hpe-system"
    SPARK = "spark-operator"
    LDAP = "hpe-ldap"
    CSI = "hpe-csi"
    MAPR_EXISTS = "kubectl get ns -o jsonpath='{.items[?(@.metadata.name == \"" + SYSTEM + "\")]}'"
    SPARK_EXISTS = "kubectl get ns -o jsonpath='{.items[?(@.metadata.name == \"" + SPARK + "\")]}'"
    LDAP_EXISTS = "kubectl get ns -o jsonpath='{.items[?(@.metadata.name == \"" + LDAP + "\")]}'"
    CSI_EXISTS = "kubectl get ns -o jsonpath='{.items[?(@.metadata.name == \"" + CSI + "\")]}'"
    CLUSTER_INFO = "kubectl get pods -n {0} -o json -l \"app={1}\""
    LDAP_INFO = "kubectl get pods -n {0} -o json -l \"hpe.com/component={1}\""
    SPARK_INFO = "kubectl get pods -n " + SPARK + " -o json -l \"app.kubernetes.io/name=sparkoperator\""

    def __init__(self):
        self._system_namespace_exists = None
        self._spark_namespace_exists = None
        self._ldap_namespace_exists = None
        self._csi_namespace_exists = None
        self._cluster_op_json = None
        self._tenant_op_json = None
        self._spark_op_json = None
        self._ldap_op_json = None
        self._csi_op_json = None

    def examine_cluster(self, wait_for_running):
        print("")
        Log.info("Gathering Data Fabric cluster information...", stdout=True)
        Log.info("Checking namespaces...", stdout=True)

        response, status = OSCommand.run2(ClusterInfo.MAPR_EXISTS)
        self._system_namespace_exists = True if status == 0 else False
        if not self._system_namespace_exists:
            Log.info("The " + ClusterInfo.SYSTEM + " namespace was not found on this Kubernetes cluster")

        response, status = OSCommand.run2(ClusterInfo.SPARK_EXISTS)
        self._spark_namespace_exists = True if status == 0 else False
        if not self._spark_namespace_exists:
            Log.info("The " + ClusterInfo.SPARK + " namespace was not found on this Kubernetes cluster")

        response, status = OSCommand.run2(ClusterInfo.LDAP_EXISTS)
        self._ldap_namespace_exists = True if status == 0 else False
        if not self._ldap_namespace_exists:
            Log.info("The " + ClusterInfo.LDAP + " namespace was not found on this Kubernetes cluster")

        response, status = OSCommand.run2(ClusterInfo.CSI_EXISTS)
        self._csi_namespace_exists = True if status == 0 else False
        if not self._csi_namespace_exists:
            Log.info("The " + ClusterInfo.CSI + " namespace was not found on this Kubernetes cluster")

        Log.info("Checking operators...", stdout=True)
        if self._system_namespace_exists:
            self._cluster_op_json = self.check_operator(wait_for_running, ClusterInfo.CLUSTER_INFO.format(ClusterInfo.SYSTEM, "dataplatformoperator"))

        if self._system_namespace_exists:
            self._tenant_op_json = self.check_operator(wait_for_running, ClusterInfo.CLUSTER_INFO.format(ClusterInfo.SYSTEM, "tenantoperator"))

        if self._spark_namespace_exists:
            self._spark_op_json = self.check_operator(wait_for_running, ClusterInfo.SPARK_INFO)

        if self._ldap_namespace_exists:
            self._ldap_op_json = self.check_operator(wait_for_running, ClusterInfo.LDAP_INFO.format(ClusterInfo.LDAP, "ldap"))

        if self._csi_namespace_exists:
            self._csi_op_json = self.check_operator(wait_for_running, ClusterInfo.CLUSTER_INFO.format(ClusterInfo.CSI, "hpe-controller-kdf"))

    def check_operator(self, wait_for_running, cmd):
        json_val = None

        for i in range(1, 20):
            response, status = OSCommand.run2(cmd)
            if status != 0:
                Log.info("Could not gather operator information")
            json_val = ClusterInfo.get_json(response)
            if not wait_for_running:
                break
            status = json_val["items"][0]["status"]["phase"].lower()
            if status == "running":
                break
            Log.info("Waiting for operators to become running (was {0})...".format(status), stdout=True)
            time.sleep(5)

        return json_val

    @staticmethod
    def get_json(json_str):
        j = json.loads(json_str)
        if j is None or len(j.get("items")) == 0:
            return None
        return j

    def get_cluster_operator_json(self):
        return self._cluster_op_json

    def get_tenant_operator_json(self):
        return self._tenant_op_json

    def get_spark_operator_json(self):
        return self._spark_op_json

    def get_ldap_operator_json(self):
        return self._ldap_op_json

    def is_data_fabric_installed(self):
        return self._system_namespace_exists and self._cluster_op_json is not None

    def is_compute_installed(self):
        return self._system_namespace_exists and self._tenant_op_json is not None

    def is_ldap_installed(self):
        return self._ldap_namespace_exists and self._ldap_op_json is not None

    def is_spark_installed(self):
        return self._spark_namespace_exists and self._spark_op_json is not None

    def is_csi_installed(self):
        return self._csi_namespace_exists and self._csi_op_json is not None

    @staticmethod
    def _get_pod_info(json_obj, title):
        index = 0

        if json_obj is None:
            return ""
        items = json_obj.get("items")
        if items is None or len(items) == 0:
            Log.info("Pod JSON has no items")
        pod_name = json_obj["items"][index]["metadata"]["name"]
        create_time = json_obj["items"][index]["metadata"]["creationTimestamp"]
        image_name = json_obj["items"][index]["spec"]["containers"][0]["image"]
        image_name = image_name[image_name.rindex("/") + 1:]
        status = json_obj["items"][index]["status"]["phase"]
        return title + os.linesep + \
            "  Pod: {0}".format(pod_name) + os.linesep + \
            "  Image: {0}".format(image_name) + os.linesep + \
            "  Create Time: {0}".format(create_time) + os.linesep + \
            "  Status: {0}".format(status)

    def __str__(self):
        rslt = os.linesep + "data fabric installed: " + str(self.is_data_fabric_installed())
        rslt += os.linesep + "compute installed: " + str(self.is_compute_installed())
        rslt += os.linesep + "ldap installed: " + str(self.is_ldap_installed())
        rslt += os.linesep + "spark installed: " + str(self.is_spark_installed())
        rslt += os.linesep + "csi installed: " + str(self.is_csi_installed())
        if self.is_data_fabric_installed():
            rslt += os.linesep + ClusterInfo._get_pod_info(self._cluster_op_json, "Data Platform Operator:")
        if self.is_compute_installed():
            rslt += os.linesep + ClusterInfo._get_pod_info(self._tenant_op_json, "Tenant Operator:")
        if self.is_ldap_installed():
            rslt += os.linesep + ClusterInfo._get_pod_info(self._ldap_op_json, "LDAP Pod:")
        if self._spark_op_json is not None:
            for i in range(0, len(self._spark_op_json["items"])):
                if self._spark_op_json["items"][i]["metadata"]["generateName"].find("sparkoperator-init-") == -1:
                    rslt += os.linesep + ClusterInfo._get_pod_info(self._spark_op_json, "Spark Operator:")
                    break

        return rslt
